AC_DEFUN(AC_YYY_CC_CXX_BEGIN, [

AC_ARG_ENABLE(debug,
[  --enable-debug          do not strip binaries but add debug info],
yyydebug=yes, yyydebug=no)

AC_ARG_ENABLE(profiling,
[  --enable-profiling      every program creates profiling output],
yyyprofiling=yes, yyyprofiling=no)

AC_ARG_ENABLE(nooptimizing,
[  --enable-nooptimizing   do not optimize],
yyynooptimizing=yes, yyynooptimizing=no)

AC_ARG_ENABLE(nowarnerror,
[  --enable-nowarnerror    do not stop make if compiler warning],
yyynowarnerror=yes, yyynowarnerror=no)


AC_ARG_WITH(cccxxpre,
[  --with-cccxxpre=x       add at beginning of C and C++ options],
yyycccxxpre="$withval",yyycccxxpre="")

AC_ARG_WITH(cccxxpost,
[  --with-cccxxpost=x      add at end of C and C++ options],
yyycccxxpost="$withval",yyycccxxpost="")


AC_ARG_WITH(ldpre,
[  --with-ldpre=x          add at beginning of linker options],
yyyldpre="$withval",yyyldpre="")

AC_ARG_WITH(ldpost,
[  --with-ldpost=x         add at end of linker options],
yyyldpost="$withval",yyyldpost="")


YYY_CFLAGS=""
YYY_CXXFLAGS=""
YYY_LDFLAGS=""

])


AC_DEFUN(AC_YYY_CC_INIT, [

AC_ARG_WITH(ccpre,
[  --with-ccpre=x          add at beginning of C options],
yyyccpre="$withval",yyyccpre="")

AC_ARG_WITH(ccpost,
[  --with-ccpost=x         add at end of C options],
yyyccpost="$withval",yyyccpost="")

# if not set then set it as empty, so that AC_PROG_CC doesn't set it
if test "$CFLAGS" = ""; then
  CFLAGS=""
fi
AC_PROG_CC

YYY_CFLAGS="$CFLAGS"

if test "$yyynooptimizing" = "yes"; then
  YYY_CFLAGS="$YYY_CFLAGS -O0"
else
  YYY_CFLAGS="$YYY_CFLAGS -O2"
fi

if test "$yyyprofiling" = "yes"; then
  yyydebug=yes
  YYY_CFLAGS="$YYY_CFLAGS -pg"
  YYY_LDFLAGS="$YYY_LDFLAGS -pg"
fi

if test "$yyydebug" = "yes"; then
  YYY_CFLAGS="$YYY_CFLAGS -g"
else
  YYY_LDFLAGS="$YYY_LDFLAGS -s"
fi

if test "$yyynowarnerror" = "yes"; then
  YYY_CFLAGS="$YYY_CFLAGS -Wall"
else
  YYY_CFLAGS="$YYY_CFLAGS -Wall -Werror"
fi

])


AC_DEFUN(AC_YYY_CXX_INIT, [

AC_ARG_WITH(cxxpre,
[  --with-cxxpre=x         add at beginning of C++ options],
yyycxxpre="$withval",yyycxxpre="")

AC_ARG_WITH(cxxpost,
[  --with-cxxpost=x        add at end of C++ options],
yyycxxpost="$withval",yyycxxpost="")

# if not set then set it as empty, so that AC_PROG_CXX doesn't set it
if test "$CXXFLAGS" = ""; then
  CXXFLAGS=""
fi
AC_PROG_CXX

YYY_CXXFLAGS="$CXXFLAGS"

if test "$yyynooptimizing" = "yes"; then
  YYY_CXXFLAGS="$YYY_CXXFLAGS -O0"
else
  YYY_CXXFLAGS="$YYY_CXXFLAGS -O2"
fi

if test "$yyyprofiling" = "yes"; then
  yyydebug=yes
  YYY_CXXFLAGS="$YYY_CXXFLAGS -pg"
  LDFLAGS="$LDFLAGS -pg"
fi

if test "$yyydebug" = "yes"; then
  YYY_CXXFLAGS="$YYY_CXXFLAGS -g"
else
  YYY_LDFLAGS="$YYY_LDFLAGS -s"
fi

if test "$yyynowarnerror" = "yes"; then
  YYY_CXXFLAGS="$YYY_CXXFLAGS -Wall"
else
  YYY_CXXFLAGS="$YYY_CXXFLAGS -Wall -Werror"
fi

])


AC_DEFUN(AC_YYY_CC_CXX_END, [

CFLAGS="$yyyccpre $yyycccxxpre $YYY_CFLAGS $yyycccxxpost $yyyccpost"
CXXFLAGS="$yyycxxpre $yyycccxxpre $YYY_CXXFLAGS $yyycccxxpost $yyycxxpost"
LDFLAGS="$yyyldpre $YYY_LDFLAGS $yyyldpost"

])
dnl Try to find the libqt1 and set ac_qt_includes and ac_qt_libraries
dnl those variables will be substituted in the makefiles.

AC_DEFUN(AC_YYY_FIND_LIBQT, [
YYY_qt_includes=NO
YYY_qt_libraries=NO
YYY_qt_moc=NO
AC_SUBST(YYY_qt_includes)
AC_SUBST(YYY_qt_libraries)
AC_SUBST(YYY_qt_moc)
YYY_try_qt_includes=""
YYY_try_qt_libraries=""
YYY_try_qt_moc=""


if test "$QTDIR" = ""; then
  true
else
  YYY_try_qt_includes="$QTDIR"/include
  YYY_try_qt_libraries="$QTDIR"/lib
  YYY_try_qt_moc="$QTDIR"/bin
fi

AC_ARG_WITH(qt-dir,
	[  --with-qt-dir           where the root of qt is installed ],
	[
		YYY_try_qt_includes="$withval"/include
		YYY_try_qt_libraries="$withval"/lib
		YYY_try_qt_moc="$withval"/bin/moc
	])
AC_ARG_WITH(qt-includes,
	[  --with-qt-includes      where the qt includes are.],
	[ YYY_try_qt_includes="$withval" ])
AC_ARG_WITH(qt-libraries,
	[  --with-qt-libraries     where the qt library is installed.],
	[ YYY_try_qt_libraries="$withval" ])
AC_ARG_WITH(qt-moc,
	[  --with-qt-moc           where the qt meta object compiler is],
	[ YYY_try_qt_moc="$withval" ])

AC_PATH_X
if test "$no_x" = "yes"; then
  AC_MSG_ERROR([X headers and library must be installed])
fi

AC_MSG_CHECKING([for libQT header files])
AC_YYY_FIND_LIBQT_FILE(qpainter.h, "$YYY_try_qt_includes" /usr/lib/qt/include /usr/local/qt/include /usr/include/qt /usr/include /usr/X11R6/include/X11/qt /usr/lib/qt1g/include, YYY_qt_includes)
if test "$YYY_qt_includes" = "NO"; then
  AC_MSG_RESULT([not found])
  AC_MSG_ERROR([libQT header files must be installed. consider ./configure --help for libQT options])
fi
YYY_qt_includes="-I$YYY_qt_includes"
AC_MSG_RESULT("$YYY_qt_includes")

AC_MSG_CHECKING([for libQT library files])
AC_YYY_FIND_LIBQT_FILE(libqt.a "libqt.so", "$YYY_try_qt_libraries" /usr/lib/qt/lib /usr/X11R6/lib /usr/lib /usr/local/qt/lib /usr/lib/qt /usr/lib/qt1g/lib, YYY_qt_libraries)
if test "$YYY_qt_libraries" = "NO"; then
  AC_MSG_RESULT([not found])
  AC_MSG_ERROR([libQT library files must be installed. consider ./configure --help for libQT options])
fi
YYY_qt_libraries="-L$x_libraries -L$YYY_qt_libraries"
AC_MSG_RESULT("$YYY_qt_libraries")

AC_MSG_CHECKING([for libQT Meta Object Compiler (moc)])
YYY_ifs_backup="$IFS"
IFS=:
AC_YYY_FIND_LIBQT_FILE(moc, "$YYY_try_qt_moc" $PATH /usr/lib/qt1g/bin, YYY_qt_moc)
IFS="$YYY_ifs_backup"
if test "$YYY_qt_moc" = "NO"; then
  AC_MSG_RESULT([not found])
  AC_MSG_ERROR([libQT meta object compiler not found. consider ./configure --help for libQT options])
fi
YYY_qt_moc="$YYY_qt_moc"/moc
AC_MSG_RESULT("$YYY_qt_moc")

AC_MSG_CHECKING([Trying to compile complete simple libQT program])
cat > conftest.cpp << EOF
#include <qpainter.h>
int main () {return 0;};
EOF
YYY_compile='${CXX} -c $CXXFLAGS $YYY_qt_includes conftest.cpp -o conftest.o'
if AC_TRY_EVAL(YYY_compile); then
  true
else
  rm -f conftest*
  AC_MSG_RESULT([error])
  AC_MSG_ERROR([hmmm. why did THIS happen ? please check config.log])
fi
AC_MSG_RESULT([ok])
AC_MSG_CHECKING([Trying to link complete simple libQT program])
YYY_link='${CXX} $LDFLAGS $YYY_qt_libraries conftest.o -o conftest -lqt -lX11 -lXext'
if AC_TRY_EVAL(YYY_link); then
  true
else
  rm -f conftest*
  AC_MSG_RESULT([error])
  AC_MSG_ERROR([hmmm. why did THIS happen ? please check config.log])
fi
AC_MSG_RESULT([ok])
rm -f conftest*

])

# find multiple files in multiple directories.
# $1 list of files (wildcards allowed)
# $2 list of directories
# $3 first directory that contained any of the files or NO if error
AC_DEFUN(AC_YYY_FIND_LIBQT_FILE,[
$3=NO
for i in $2; do
  for j in $1; do
    for k in "$i/"$j; do
      if test -r "$k" && test -f "$k"; then
        $3=$i
        break 3
      fi
    done
  done
done
])
