/*
  Memory allocation algorithms.
  Copyright (C) 1999  Hardy Kahl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ----------------------------------------------------------------------------
//  Description      : Memory allocation algorithms.
// ----------------------------------------------------------------------------
//  Remarks          :
//    Hardy     12.11.1998      nobody should change this file, except the
//                              whole team decides.
// ----------------------------------------------------------------------------
//  Modifications    :
//    Hardy     12.11.1998      created this file.
// ----------------------------------------------------------------------------




#ifndef AUTO_H
#define AUTO_H




#include "base.h"
#include "exgen.h"
#include "numeric.h"




template <class T>
class xAllocator
{
  protected:
  T *Object;

  public:
  xAllocator()
    {
      Object = new T;
      if (!Object) throw xMemoryException(ECMEM_GENERAL,sizeof(T));
    }
  xAllocator(T *object)
    : Object(object)
    { }
  ~xAllocator()
    { if (Object) delete Object; }
  operator T &()
    {
      if (!Object) throw xGenericException(EC_NULLPOINTER);
      return *Object;
    }
  operator T *() const
    { return Object; }
  operator void *() const
    { return (void *) Object; }

  T *operator!() const
    { return Object; }

  T *operator=(xAllocator<T> &allocator)
    {
      if (Object) delete Object;
      Object = new T;
      if (!Object) throw xMemoryException(ECMEM_GENERAL,sizeof(T));
      *Object = (T)allocator;
    }
};




template <class T>
class xArrayAllocator
{
  protected:
  TSize Count;
  T *Object;

  public:
  xArrayAllocator(TSize count = 0)
    : Count(count)
    {
      if (count) {
        Object = new T[count];
        if (!Object) throw xMemoryException(ECMEM_GENERAL,sizeof(T)*count);
      }
      else Object = NULL;
    }
  xArrayAllocator(T *object,TSizeD count)
    : Count(count), Object(object)

    { }
  ~xArrayAllocator()
    { if (Object) delete[] Object; }
  void operator()(TSize count = 0)
    {
      if (count) {
        T *object = new T[count];
        if (!object) throw xMemoryException(ECMEM_GENERAL,sizeof(T)*count);

        TSize copynum = NUM_MIN(count,Count);

        for (TIndex index = 0;index < copynum;index++)
          object[index] = Object[index];
        if (Object) delete[] Object;

        Object = object;
      }
      else {
        delete[] Object;
        Object = NULL;
      }
      Count = count;
    }
  operator T *() const
    { return Object; }
  operator void *() const
    { return (void *) Object; }
  T &operator[](TIndex index) const
    {
      if (index >= Count) EX_THROW(Generic,EC_INDEX)
      return Object[index];
    }
  T *operator!() const
    { return Object; }
  TSize operator~() const
    { return Count; }

  T *operator=(xArrayAllocator<T> &allocator)
    {
      if (Object) delete[] Object;

      if (~allocator) {
        Object = new T[~allocator];
        if (!Object) throw xMemoryException(ECMEM_GENERAL,sizeof(T)*(~allocator));
      }
      else Object = NULL;

      for (TIndex i = 0;i < ~allocator;i ++) {
        Object[i] = allocator[i];
      }

      return Object;
    }
};




#endif
