/*
  Generic Exceptions
  Copyright (C) 1999  Hardy Kahl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ----------------------------------------------------------------------------
//  Description      : Generic exceptions
// ----------------------------------------------------------------------------
//  (c) Copyright 1996 by iXiONmedia, all rights reserved.
// ----------------------------------------------------------------------------




#include "numeric.h"
#include "xstring.h"




// Plain text rendering table -------------------------------------------------
#define T_CANNOTEVALUATE      "Unable to evaluate expression"
#define T_NOTYETIMPLEMENTED   "Function not yet implemented"
#define T_ERROR               "General error"
#define T_NULLPOINTER         "NULL pointer encountered"
#define T_INVALIDPAR	      "Invalid parameter"
#define T_INDEX               "Index out of range"
#define T_BUFFEROVERFLOW      "Buffer overrun"
#define T_BUFFERUNDERFLOW     "Buffer underrun"
#define T_ITEMNOTFOUND        "Item not found"
#define T_INVALIDOP           "Invalid operation"

#define TMEM_GENERAL          "Allocation failed"
#define TMEM_CONVENTIONAL     "Allocation (conv) failed"
#define TMEM_EXTENDED         "Allocation (XMS) failed"
#define TMEM_EMS              "Allocation (EMS) failed"
#define TMEM_SWAP             "Not enough swap space"




static char *(PlainText[]) =
  { T_CANNOTEVALUATE,T_NOTYETIMPLEMENTED,T_ERROR,
    T_NULLPOINTER,T_INVALIDPAR,T_INDEX,T_BUFFEROVERFLOW,T_BUFFERUNDERFLOW,
    T_ITEMNOTFOUND,T_INVALIDOP };
static char *(MemoryPlainText[]) =
  { TMEM_GENERAL,TMEM_CONVENTIONAL,TMEM_EXTENDED,TMEM_EMS,TMEM_SWAP };




// xGenericException ----------------------------------------------------------
char *xGenericException::GetText() const
{
  return PlainText[Error];
}




// xMemoryException -----------------------------------------------------------
xMemoryException::xMemoryException(TErrorCode error,TSizeD bytesmissing,
  char *module,TIndexD line)
: xBaseException(error,NULL,module,line,"MEM")
{
  if (bytesmissing) {
    HasInfo = TRUE;
    try {
      strcpy(Info,xString::Bytes2Dec(bytesmissing));
    }
    catch (...) { }
  }
};




char *xMemoryException::GetText() const
{
  return MemoryPlainText[Error];
}

