/*
  Numeric / order processing
  Copyright (C) 1999  Hardy Kahl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ----------------------------------------------------------------------------
//  Description      : Numeric / order processing
// ----------------------------------------------------------------------------
//  Remarks          :
//    CircleDist(): "head" is where the circle FIFO grows.
//
// ----------------------------------------------------------------------------
//  (c) Copyright 1996 by iXiONmedia, all rights reserved.
// ----------------------------------------------------------------------------




#ifndef NUMERIC_H
#define NUMERIC_H




#include <string.h>
#include "base.h"
#include "exgen.h"




// Macros ---------------------------------------------------------------------
#define NUM_COMPARE(item,refitem) \
  ( item<refitem ? RBelow : \
    item>refitem ? RAbove : REqual )
#define NUM_MIN(a,b) \
  ( a<b ? a : b )
#define NUM_MAX(a,b) \
  ( a>b ? a : b )
#define NUM_LIMIT(lower,value,upper) \
  ( value<lower ? lower : \
    value>upper ? upper : value )
#define NUM_INBOUND(lower,value,upper) \
  ((lower <= value) && (value <= upper))
#define NUM_OVERLAP(a1,a2,b1,b2) \
  (((a1<=b1)&&(a2>b1))||((a1<b2)&&(a2>b2))||((a1>=b1)&&(a2<=b2)))
#define NUM_CIRCLEINC(index,size) \
  ( index+1 >= size ? 0 : index+1 )
#define NUM_CIRCLEDIST(head,tail,size) \
  ( head<tail ? head+size-tail : head-tail )

// Primitive inlines ---------------------------------------------------------
inline signed short NumSgn(signed long value);
inline TBool numGetBit(unsigned long value,char bit);
inline TByte numHiByte(TWord value);
inline TWord numHiWord(TDWord value);
inline TByte numLoByte(TWord value);
inline TWord numLoWord(TDWord value);
inline TWord numMakeWord(TByte hi,TByte lo);
inline TDWord numMakeDWord(TWord hi,TWord lo);
inline ERelation numSlight(ERelation rel);

// Primitive inline templates -------------------------------------------------
template<class T>
inline void numSwap(T &a,T &b);

// BCD encoding ---------------------------------------------------------------
unsigned long numUnsigned2BCD(unsigned long value);
unsigned long numBCD2Unsigned(unsigned long value);




// Primitive inlines ---------------------------------------------------------
inline signed short numSgn(signed long value)
{
  return (value<0) ? -1 : ( (value>0) ? 1 : 0);
}




inline TBool numGetBit(unsigned long value,char bit)
{
  return (value >> bit) & 1;
}




inline TByte numHiByte(TWord value)
{
  return value >> 8;
}




inline TWord numHiWord(TDWord value)
{
  return value >> 16;
}




inline TByte numLoByte(TWord value)
{
  return value & 0xff;
}




inline TWord numLoWord(TDWord value)
{
  return value & 0xffff;
}




inline TWord numMakeWord(TByte hi,TByte lo)
{
  return (TWord) hi << 8 | lo;
}




inline TDWord numMakeDWord(TWord hi,TWord lo)
{
  return (TDWord) hi << 16 | lo;
}




inline ERelation numSlight(ERelation rel)
{
  return (ERelation) (rel / 2);
}




// Primitive inline templates -------------------------------------------------
template<class T>
inline void numSwap(T &a,T &b)
{
  T temp = a;
  a = b;
  b = temp;
}




#endif
