/*
  RAM registers
  Copyright (C) 1999  Hardy Kahl

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ----------------------------------------------------------------------------
//  Description      : RAM registers.
// ----------------------------------------------------------------------------
//  Remarks          :
//    Hardy     12.11.1998      nobody should change this file, except the
//                              whole team decides.
//    Hardy     01.04.1999      xRegisterManager is a dynamically growing list,
//                              that stores register entries (xNamedRegister).
//                              To select a special register an index ist
//                              stored together with register entry.
//    Hardy     01.04.1999      operator [] accepts xNamedRegister or xString
//                              as an index.
//                              To get the real value of register entry, use
//                              the GetValue(). It reverses the value, so
//                              you get the right number.
// ----------------------------------------------------------------------------
//  Modifications    :
//    Hardy     12.11.1998      created this file.
//    Hardy     16.11.1998      changed TIndex to xString. Now R[R[10]]
//                              possible.
// ----------------------------------------------------------------------------




#ifndef REGISTER_H
#define REGISTER_H




#include "ctlist.h"
#include "xstring.h"




class xNamedRegister
{
  protected:
  xString Content;
  xString Name;

  public:
  xNamedRegister(xString name);

  TBool IsZero()
    { return Content == "0"; }

  xNamedRegister operator=(char const *value)
    { return Set(xString(value).Reverse()); }
  xNamedRegister operator=(xString const &value)
    { return Set(value.Reverse()); }
  xNamedRegister operator=(xNamedRegister const &value)
    { return Set(value); }
  xNamedRegister operator+(xNamedRegister const &value)
    { return Add(value); }
  xNamedRegister operator-(xNamedRegister const &value)
    { return Sub(value); }
  xString operator()() const
    { return Content; }

  xString GetName() const               // register number
    { return Name; }
  xString GetValue() const              // content in right order for output
    { return Content.Reverse(); }

  private:
  xNamedRegister Set(xString const &value);
  xNamedRegister Set(xNamedRegister const &reg);
  xNamedRegister Add(xNamedRegister const &reg);
  xNamedRegister Sub(xNamedRegister const &reg);

  char AddDigits(char digit1, char digit2, char &carry);
  char SubDigits(char digit1, char digit2, char &carry);
  void Optimize(xString &number);
};




class xRegisterList : public xList<xNamedRegister *>
{
  private:
  virtual void Kill(xNamedRegister *&object);
};




class xRegisterManager 
{
  public:
  xList<xNamedRegister *> RegisterList;

  xNamedRegister &operator[](xString const &index)
    { return FindReg(index); }
  xNamedRegister &operator[](xNamedRegister const &index)
    { return FindReg(index.GetValue()); }

  private:
  xNamedRegister &FindReg(xString const &index);
};




#endif
