/*
  plic lexer
  Copyright (C) 1999  Philipp Cimiano Lavin

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

%{
/* here follows a lexical analyzer to be used by the higher-level */
/* Pascalli-parser                                                */

#define YY_NO_UNPUT

#include "frontend.h"
#include "pascalli_parser.h"
#include <ctype.h>
#include <string.h>
#include <iostream.h>


char *Line;
// contains the line

int Line_Number=0;
// for better error reporting!

int TokenPosition=1;
// for still better error reporting!

%}

%%

\n.*     {
	   Line=strdup(yytext+1);
	   Line_Number++;
	   TokenPosition=1;
	   yyless(1);
	 }


function {
           TokenPosition=TokenPosition+yyleng;
	   return FUNCTION;
	 }

do	 {
           TokenPosition=TokenPosition+yyleng;
	   return DO;
	 }
while    {
	   TokenPosition=TokenPosition+yyleng;
	   return WHILE;
	 }

if       {
           TokenPosition=TokenPosition+yyleng;
	   return IF;
	 }

then	 {
           TokenPosition=TokenPosition+yyleng;
	   return THEN;
	 }

else	 {
	   TokenPosition=TokenPosition+yyleng;
	   return ELSE;
	 }

for  	 {
           TokenPosition=TokenPosition+yyleng;
	   return FOR;
	 }

to       {
	   TokenPosition=TokenPosition+yyleng;
	   return TO;
	 }

and	 {
	   TokenPosition=TokenPosition+yyleng;
	   return AND;
	 }

not      {
	   TokenPosition=TokenPosition+yyleng;
	    return NOT;
	 }

or	 {
	   TokenPosition=TokenPosition+yyleng;
	   return OR;
	 }

begin {
	TokenPosition=TokenPosition+yyleng;
	return BEG;
      }

end {
      TokenPosition=TokenPosition+yyleng;
      return END;
    }

[0-9]+ {
	 yylval.tConstant=strdup(yytext);
	 TokenPosition=TokenPosition+yyleng;
	 return CONSTANT;
       }

[a-zA-Z][a-zA-Z0-9]* {
		      yylval.tIdentifier=strdup(yytext);
		      TokenPosition=TokenPosition+yyleng;
		      return IDENTIFIER;
		     }


[\t ] {
        TokenPosition=TokenPosition+yyleng;
      };


. {
    TokenPosition=TokenPosition+yyleng;
    return yytext[0];
  }

%%


void yyerror(char *s)
{
    cout << "At line " << Line_Number <<": " << s << "\n";
    cout <<"\t" << Line << "\n";
    printf("\t%*s\n",TokenPosition,"*^*");
}


void yyerror2(char *s)
{
  cout << "At line " << Line_Number <<": " << s << "\n";
};











