/*
  plic parser
  Copyright (C) 1999  Philipp Cimiano Lavin

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// This is the implementation of a Yacc-Parser for Pascalli.

%{

#include <stdio.h>
#include <time.h>
#include <iostream.h>
#include <stdlib.h>
#include "frontend.h"
#include "symboltable.h"
#include "plic_Translator.h"
#include "base.h"
#include "xstring.h"
#include "xset.h"



int yylex();

void yyerror(char *s);

void yyerror2(char *s);

TBool used_but_not_declared();

void message_compilation_finished();

void message_abnormal_compilation();

void free_parstree();


// the TopNode
xProgramNode *TopNode;

// this variable contains the name of the function being parsed
xIdentifier *Function_Name;

// this flag is set TRUE if the function has a return-value
TBool Value_Assigned=FALSE;

int Status;


// a empty Symboltable for resetting is created
xIdentifierSet New_Symboltable;

// the Symboltable for the variables is created
xIdentifierSet Variable_Symboltable;

// the Symboltable for the array is created
xIdentifierSet Array_Symboltable;

// the Symboltable for the declared functions is created
xIdentifierSet Declared_Functions_Symboltable;

// the Symboltable for the used functions is created
xIdentifierSet Used_Functions_Symboltable;

// the the Symboltable for the local functions is created
xIdentifierSet Local_Functions_Symboltable;

// the list containing the xIdentifierList to free is created
List_Of_IdentifierLists Free_IdentifierLists;

// the list containing the xExpressionList to free is created
List_Of_ExpressionLists Free_ExpressionLists;

// the list containing the xFunctionList to free is created
List_Of_FunctionLists Free_FunctionLists;

// the list containing the xCommandList to free is created
List_Of_CommandLists Free_CommandLists;

// the list containing the xIdentifiers to free is created
List_Of_Nodes Free_Identifiers;

// the list containing the xNodes to free is created
List_Of_Nodes Free_Nodes;

TIndex Command_Counter=1;

%}

/* The section until the first '%%' is called the Definition Section.  */
/* In this section the valid TOKENS returned by the Lexer are declared */
/* and assigned to a type.                                             */
/* Also the NON-TERMINALS are declared and assigned to a type.         */

%union
{
  char          *Key_word;
  char          *tIdentifier;
  char          *tConstant;
  xIdentifierList *tIdentifierList;
  xExpressionList *tExpressionList;
  xProgramNode  *tProgram;
  xFunction     *tFunction;
  xArray        *tArray;
  xCommandBlock *tCommandBlock;
  xCommand      *tCommand;
  xAssignment   *tAssignment;
  xIfStatement *tIf;
  xForLoop      *tFor;
  xWhile        *tWhile;
  xExpression   *tExpression;
  xCondition    *tCondition;
}

%token <Key_word>    FUNCTION
%token <Key_word>    BEG
%token <Key_word>    END
%token <tIdentifier> IDENTIFIER
%token <tConstant>   CONSTANT
%token <Key_word>    WHILE
%token <Key_word>    DO
%token <Key_word>    IF
%token <Key_word>    THEN
%token <Key_word>    ELSE
%token <Key_word>    FOR
%token <Key_word>    TO
%token <Key_word>    NOT
%token <Key_word>    OR
%token <Key_word>    AND

/* The tokens ';' ',' '#' ':' '=' '(' ')' don't need to be declared.  */


%type <tProgram>      program;
%type <tFunction>     function;
%type <tArray>        array;
%type <tIdentifierList>      arglist;
%type <tExpressionList>      arglist2;
%type <tCommandBlock> commandblock;
%type <tCommand>      command;
%type <tAssignment>   assignment;
%type <tIf>           if_command;
%type <tFor>          for_command;
%type <tWhile>        while_command;
%type <tExpression>   expression;
%type <tCondition>    condition;

%nonassoc LOWER_THAN_ELSE 
%nonassoc ELSE

%%

/* Here begins the Rule Section.                                      */


program: program function
	 {
	   TIndex index;
	   xFunctionList *functionlist;
	   xProgramNode  *programnode;
	   xNode         *node;
	   
	   // a new xFunctionList is created
	   functionlist=new xFunctionList;

	   // getting the functionlist
	   (*functionlist)=$1->Functions;

	   // the function is inserted into the functionlist
	   (*functionlist)+=$2;
	   
	   programnode=$1;

	   programnode->Functions=(*functionlist);

	   node=$2;
	   
	   index=Free_Nodes(node);
	   Free_Nodes-=index;
	   
	   // functionlist must be freed!
	   Free_FunctionLists+=functionlist;

	   TopNode=programnode;
	   
	   $$=programnode;
	   
	   // this rule raises normal syntax-error!!!
	   
	   // cout <<"Program parsed!!!\n";
	 }
       | function
         {
	   TIndex        index;
	   xFunctionList *functionlist;
           xProgramNode  *programnode;
	   xNode         *node;

	   // a new xFunctionList is created
	   functionlist= new xFunctionList;

	  // the function is inserted into the functionlist
          (*functionlist)+=$1;

	  // the xProgramNode to be returned is created
	  programnode=new xProgramNode((*functionlist));

	  // functionlist must be freed!
	  Free_FunctionLists+=functionlist;

	  Free_Nodes+=programnode;

	  node=$1;

	  index=Free_Nodes(node);
	  Free_Nodes-=index;

	  $$=programnode;
	  
	  TopNode=programnode;

	  // this rule raises normal syntax-error!!!

	  // cout << "first function parsed!!!" <<endl;
	 }
       ;

function: FUNCTION IDENTIFIER
	  { 
	    xString symbol;
	    symbol=xString($2);
	    Function_Name=new xIdentifier(symbol);
	    Function_Name->Type=Function;

	    Free_Identifiers+=Function_Name;

	    // checking if the identifier is already in the Declared...
	    if (Declared_Functions_Symboltable.Contains(Function_Name))
	      // the identifier is a already declared function
	     	{
		  yyerror("Function already declared!!!");
	     	  return 1;
		}
	    
	    // the function-identifier is inserted into the 
	    // Declared_Functions_Symboltable and into the
	    // Local_Functions_Symboltable as an xIdentifier
	    Local_Functions_Symboltable+=Function_Name;
	    Declared_Functions_Symboltable+=Function_Name;
	  }

          '(' arglist ')' ';' command
          {
	    xNode       *node; 
	    TIndex      index;
	    xFunction   *function;

	   // the Variable_Symboltable and the Array_Symboltable are reseted
	    Variable_Symboltable = New_Symboltable;
	    Array_Symboltable    = New_Symboltable;

	    Function_Name->Number_Of_Arguments=~(*$5);

	   // and after being freed it is reseted
	   Local_Functions_Symboltable = New_Symboltable;

	   // checking if the function has a return-value
	   if (Value_Assigned==TRUE)
	     {
	       Value_Assigned=FALSE;
	     }
	   else
	     {
	       yyerror("Function has no return-value!!!");
	       return 1;
	     }

	   // the xFunction to be returned is created
	   function=new xFunction(Function_Name,(*$5),$8);

	   Free_Nodes+=function;
	   
	   for (TIndex i=0; i<~(*$5); i++)
	     {
	       node=(*$5)[i];
	       index=Free_Nodes(node);
	       Free_Nodes-=index;
	     }


	   node=$8;

	   index=Free_Nodes(node);
	   Free_Nodes-=index;

      	   $$=function;
	   
	   // cout <<"Function parsed!!!\n";
	  }
        ;

commandblock: commandblock ';' command
              {
		TIndex        index;
		xCommandBlock *commandblock;
		xCommandList  *commandlist;
		xNode         *node;

		// a xCommandList is created
		commandlist= new xCommandList;

		(*commandlist)=$1->CommandList;
	      
		// the command is inserted into the commandlist
		(*commandlist)+=$3;

		//commandlist must be freed!
		Free_CommandLists+=commandlist;

		commandblock=$1;

		commandblock->CommandList=(*commandlist);
		commandblock->Alpha=($1->Alpha);
		commandblock->Beta=($3->Beta);

		node=$3;

		index=Free_Nodes(node);
		Free_Nodes-=index;
		
		$$=commandblock;

		// cout << "Commandblock parsed!\n";

		// this rule raises only normal syntax-error!!!
	      }
	    | command
	      {
		TIndex        index;
		xCommandBlock *commandblock;
		xCommandList  *commandlist;
		xNode         *node;

		// the commandlist is created
		commandlist = new xCommandList;

		// the command is inserted into the commandlist
		(*commandlist)+=$1;

		// a xCommandBlock is created out of the commandlist
		commandblock=new xCommandBlock((*commandlist));

		Free_Nodes+=commandblock;

		//commandlist must be freed!
		Free_CommandLists+=commandlist;

		commandblock->Alpha=($1->Alpha);
		commandblock->Beta=($1->Beta);

		node=$1;

		index=Free_Nodes(node);
		Free_Nodes-=index;

		$$=commandblock;

		// cout << "Begin of Commandblock parsed!!!\n";
	      }
	    ;
		     

arglist: arglist ',' IDENTIFIER
	 {
	   xString   symbol;
	   xIdentifierList  *arglist;
	   xVariable *variable;

	   // a xString is created out of IDENTIFIER
	   symbol= xString($3);

	   // a xVariable is created
	   variable=new xVariable(symbol);

	   Free_Nodes+=variable;

	   // checking if the variable is already in the Symboltable
	   if (Variable_Symboltable.Contains(variable))
	     {
	       yyerror("Formal parameter used before!!!");
	       return 1;
	     }

	   // checking if the variable is the function_identifier
	   if ((*variable)==(*Function_Name))
	     {
	       yyerror("Function-identifier not allowed as formal parameter!!!");
	       return 1;
	     }

	   // the variable is inserted into the arglist
	   arglist=$1;
	   (*arglist)+=variable;

	   $$=arglist;

	   // cout <<"Arglist parsed!!!";

	   // this rule raises an error if the formal parameter
	   // was used before.
	 }

       | IDENTIFIER
	 {
	   xString          symbol;
	   xIdentifierList  *arglist;
	   xVariable        *variable;
	   
	   // a xString is created out of IDENTIFIER
	   symbol=xString($1);

	   // a xVariable is created
	   variable=new xVariable(symbol);

	   Free_Nodes+=variable;

	   // checking if the variable is already in the Variable_Symboltable
	   if (Variable_Symboltable.Contains(variable))
	     {
	       yyerror("Formal parameter used before!!!");
	       return 1;
	     }

	   // checking if the variable is the function_identifier
	   if ((*variable)==(*Function_Name))
	     {
	       yyerror("Function-identifier not allowed as formal parameter!!!");
	       return 1;
	     }
	  	   
	   // the variable is inserted into the Variable_Symboltable
	   Variable_Symboltable+=variable;

	   // a new xArgList is created
	   arglist= new xIdentifierList;

	   // the arglist must bee freed!
	   Free_IdentifierLists+=arglist;

	   
	   // the variable is inserted into the arglist
	   (*arglist)+=variable;

	   $$=arglist;
	   
	   // cout <<"Begin of Arglist parsed!!!";

	   // this rule raises an error if the formal parameter
	   // was used before.
	 }
       | 
 	 {
	   xIdentifierList *arglist;

	   // a xArgList is created to be returned
	   arglist= new xIdentifierList;

	   $$=arglist;

	   // cout <<"Empty Arglist parsed!!!";

	   // this rule raises only normal syntax-error!!!
	 }	
       ;


arglist2: arglist2 ',' expression
	 {
	   xExpressionList *arglist;

	   arglist=$1;

	   // the expression is inserted into the arglist
	   (*arglist)+=$3;
	   
	   $$=arglist;

	   // this rule raises only normal syntax-error!!!
	   
	 }

       | expression
	 {
	   xExpressionList *arglist;

	   // a xArgList is created
	   arglist=new xExpressionList;

	   // the expression is inserted into the arglist
	   (*arglist)+=$1;

	   $$=arglist;

	   // cout <<"End of Arglist parsed!!!\n";

	   // this rule raises normal syntax-error!!!
	   
	 }
       | 
 	 {
	   xExpressionList *arglist;

	   // a empty xArgList is created
	   arglist=new xExpressionList;

	   $$=arglist;

	   // cout <<"Empty Arglist parsed!!!\n";

	   // this rule raises normal syntax-error!!!
	   
	 }	
       ;

command: BEG commandblock END
	 {
	   // the commandblock is returned
	   $$=$2;

	   // cout <<"BEGIN...END-Commandblock parsed!!!\n";

	   // this rule raises normal syntax-error!!!
	 }
       | assignment
	 {
  	   // the xAssignment is returned
	   $$=$1;

	   // cout <<"Assigment parsed!!!\n";

	   Command_Counter++;

	   // this rule raises only normal syntax-error!!!
	 }
       | if_command
	 {
	   // the xIfStatement is returned
	   $$=$1;

	   // this rule raises only normal syntax-error!!!
	 }
       | for_command
	 {
	   // the xForLoop is returned
	   $$=$1;

	   // this rule raises only normal syntax-error!!!
	 }
       | while_command
	 {
	   // the xWhile is returned
	   $$=$1;

	   // this rule raises only normal syntax-error!!!
	 }
       ;
	  
       
assignment: IDENTIFIER ':' '=' expression
	   {
	     TIndex index;
	     xNode       *node;
	     xString     symbol;
	     xVariable   *variable;
	     xAssignment *assignment;

	     // a xString is created out of IDENTFIER
	     symbol=xString($1);

	     // a xVariable is created
	     variable= new xVariable(symbol);

	     Free_Nodes+=variable;

	     variable->Alpha=Command_Counter;
	     variable->Beta=Command_Counter;

	     // checking if the variable is already in the Array_Symboltable
	     if (Array_Symboltable.Contains(variable))
	       // the variable is in the Array_Symboltable
	       {
		 yyerror("Variable used as an array before!!!");
		 return 1;
	       }

	     if (!((*Function_Name)==(*variable)))
	       {
		 // checking if the variable has been used as function-identifier
		 if (Local_Functions_Symboltable.Contains(variable))
		   {
		     yyerror("Variable used as a function before!!!");
		     return 1;
		   }

		  // the variable is inserted into the Variable_Symboltable
	   	  Variable_Symboltable+=variable;
	       }

	     // the variable has not been used as an array
	     // nor a function before 
	     // a expcetion to this is that the variable is 
	     // the Function_Name

	    
	     //the xAssignment to be returned is created
	     assignment = new xAssignment(variable,$4);

	     Free_Nodes+=assignment;

	     assignment->Alpha=Command_Counter;
	     assignment->Beta=Command_Counter;

	     node=variable;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;
	     
	     node=$4;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     //checking if the variable is the function-identifier
	     
	     if ((*variable)==(*Function_Name))
	       {
		 Value_Assigned=TRUE;
	       }

	     $$=assignment;
	     
	     // cout <<"expression-assignment parsed!!!\n"; 
	     
	     // this rule raises an error if the variable has been used
             // as an array or as a function before!
	   }
	 | array ':' '=' expression
	   {
	     TIndex index;
	     xAssignment *assignment;
	     xNode       *node;

	     // a xAssignment is created to be returned
	     assignment= new xAssignment($1,$4);

	     Free_Nodes+=assignment;

	     assignment->Alpha=Command_Counter;
	     assignment->Beta=Command_Counter;

	     node=$1;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;
	     
	     $$=assignment;

	     // cout <<"array-assignment parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 ;
	
if_command: IF condition THEN command ELSE command
	    {
	      TIndex index;
	      xIfStatement *if_else_command;
	      xNode        *node;
	      
	      // a xIfStatement to be returned is created
	      if_else_command= new xIfStatement($2,$4,$6);

	      Free_Nodes+=if_else_command;

	      if_else_command->Alpha=$4->Alpha;
	      if_else_command->Beta=$6->Beta;

	      $2->Alpha=if_else_command->Alpha;
	      $2->Beta=if_else_command->Beta;

	      node=$2;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      node=$4;
	      
	      index=Free_Nodes(node);
	      Free_Nodes-=index;
	      
	      node=$6;
	      
	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      $$=if_else_command;

	      // cout <<"if-else-command parsed!!!\n";

	       // this rule raises only normal syntax-error!!!
	    }
	  | IF condition THEN command %prec LOWER_THAN_ELSE
	    {
	      TIndex index;
	      xIfStatement *if_command;
	      xNode        *node;

	      // a xIfStatement to be returned is created
	      if_command= new xIfStatement($2,$4,0);

	      Free_Nodes+=if_command;

	      if_command->Alpha=$4->Alpha;
	      if_command->Beta=$4->Beta;

	      $2->Alpha=if_command->Alpha;
	      $2->Alpha=if_command->Beta;

	      node=$2;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;
	      
	      node=$4;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;
	      
	      $$=if_command;
	      
	      // cout <<"if-command parsed!!!\n";

	      // this rule raises only normal syntax-error!!!
	    }
	  ;


for_command: FOR IDENTIFIER ':' '=' expression TO expression DO command
	     {
	       TIndex    index;
	       xString   symbol;
	       xForLoop  *for_command;
	       xVariable *counter;
	       xNode     *node;
	       
	       // a xString is created out of IDENTIFIER
	       symbol=xString($2);

	       // a xVariable is created
	       counter= new xVariable(symbol);
	       counter->Alpha=$9->Alpha;
	       counter->Beta=$9->Beta;

	       Free_Nodes+=counter;

	       // checking if the counter is in the Array_Symboltable
	       if (Array_Symboltable.Contains(counter))
		 {
		   yyerror("Variable used as an array before!!!");
		   return 1;
		 }

	       // checking if the counter has been used as a function-identfier
	       if (Local_Functions_Symboltable.Contains(counter))
		 {
		   yyerror("Variable used as a function before!!!");
		 }

	       // checking if the counter is used at the left side
	       // of an assignment in command
		   
	       if($9->variable_used(counter))
		 {	
		   yyerror("Counter-Variable used at the left side of an assignment!!!");
		   return 1;		     
		 }
	       
	       // the counter is inserted into the Variable_Symboltable
	       Variable_Symboltable+=counter;
	       
	       // a xForLoop is created to be returned
	       for_command= new xForLoop(counter,$5,$7,$9);

	       Free_Nodes+=for_command;

	       for_command->Alpha=counter->Alpha;
	       for_command->Beta=counter->Beta;

	       $5->Alpha=for_command->Alpha;
	       $5->Beta=for_command->Beta;

	       $7->Alpha=for_command->Alpha;
	       $7->Beta=for_command->Beta;

	       node=counter;
	       
	       index=Free_Nodes(node);
	       Free_Nodes-=index;

	       node=$5;

	       index=Free_Nodes(node);
	       Free_Nodes-=index;

	       node=$7;

	       index=Free_Nodes(node);
	       Free_Nodes-=index;

	       node=$9;

	       index=Free_Nodes(node);
	       Free_Nodes-=index;

	       $$=for_command;
	       
	       // cout <<"for-command parsed!!!\n";

	       // this rule raises an error if the Counter-Variable
	       // is used at the left side of an assignment in command
	       // or if the counter has been used as an array before
	     }
	   ;


while_command: WHILE condition DO command
	       {
		 TIndex index;
		 xWhile  *while_command;
		 xNode   *node;

		 // a xWhile is created to be returned
		 while_command = new xWhile($2,$4);

		 Free_Nodes+=while_command;
		 
		 node=$2;

		 index=Free_Nodes(node);
		 Free_Nodes-=index;
		 
		 node=$4;

		 index=Free_Nodes(node);
		 Free_Nodes-=index;

		 while_command->Alpha=$4->Alpha;
		 while_command->Beta=$4->Beta;

		 $2->Alpha=while_command->Alpha;
		 $2->Beta=while_command->Beta;

		 $$=while_command;
		 
	         // cout <<"while-command parsed!!!\n";

		 // this rule raises normal syntax-error!!!
	       }
	     ;

expression: CONSTANT
	    {
	      xString   number;
	      xConstant *constant;
	      
	      // a xString is created out of CONSTANT
	      number= xString($1);

	      // a xConstant is created
	      constant= new xConstant(number);

	      Free_Nodes+=constant;

	      constant->Alpha=Command_Counter;
	      constant->Beta=Command_Counter;

	      $$=constant;

	      // this rule raises normal syntax-error!!!

	      // cout <<"constant-expression parsed!!!\n";
	    }
	  | IDENTIFIER
	    {
	      xString   symbol;
	      xVariable *variable;
	 
	      // a xString is created out of IDENTIFIER
	      symbol=xString($1);

	      // a xVariable is created
	      variable=new xVariable(symbol);

	      Free_Nodes+=variable;

	      // checking if the variable is the function_identifier
	      if (*(variable)==(*Function_Name)) 
		{
		  yyerror("Function-identifier used at right side of an assignment!!!");
		  return 1;
		}


	      //checking if the variable is in the Array_Symbltable
	      if (Array_Symboltable.Contains(variable))
		{ 
		  yyerror("Variable used as an array before!!!");
		  return 1;
		}

	      if (Local_Functions_Symboltable.Contains(variable))
		{
		  yyerror("Variable used as a function before!!!");
		  return 1;
		}
								

	      variable->Alpha=Command_Counter;
	      variable->Beta=Command_Counter;

	      // the variable is inserted into the Variable_Symboltable
	      Variable_Symboltable+=variable;

	      $$=variable;

	      // cout <<"variable-expression parsed!!!\n";

	      // this rule raises an error if the variable has been used as
	      // an array before and if the identifier is the function-identifier
	    }
	  | array
          { 	    
	    $$=$1;	  
	    
	    // cout <<"array-expression parsed!!!\n";
	    
	    //this rule raises only normal syntax-error!!!
	    }

	  | IDENTIFIER '(' arglist2 ')'
	    {
	      TIndex         index;
	      xString        symbol;
	      xIdentifier    *identifier;
	      xFunctionCall  *call;
	      xNode          *node;

	      // a xString is created out of IDENTIFIER
	      symbol=xString($1);

	      // a xIdentifier is created out of the identifier
	      identifier=new xIdentifier(symbol);

	      identifier->Number_Of_Arguments=~(*$3);

	      Free_Identifiers+=identifier;

	      // checking if the identifier has been used as a variable
	      if (Variable_Symboltable.Contains(identifier))
		{
		  yyerror("Function used as a variable before!!!");
		  return 1;
		}

	      // checking if the identifier has been used as an array
	      if (Array_Symboltable.Contains(identifier))
		{
		  yyerror("Function used as an array before!!!");
		  return 1;
		}

	      // the idenitifier is inserted into the Local_Functions_Symboltable
	      Local_Functions_Symboltable+=identifier;
	      
	      // and also into the Used_Functions_Symboltable
	      Used_Functions_Symboltable+=identifier;

	      // a xFunctionCall is created
	      call=new xFunctionCall(symbol,(*$3));

	      Free_Nodes+=call;

	      //arglist2 must be freed!
	      Free_ExpressionLists+=$3;
	      
	      call->Alpha=Command_Counter;
	      call->Beta=Command_Counter;

	      for (TIndex i=0; i<~(*$3); i++)
		{
		  node=(*$3)[i];
		  index=Free_Nodes(node);
		  Free_Nodes-=index;
		}

	      $$=call;

	      // cout <<"function-call parsed!!!\n";

	       // this rule raises an error if the function identifier
	       // has been used as a variable or an array before.
	    }


	  | '(' expression '+' expression ')'
	    {

	      TIndex    index;
	      xAddition *addition;
	      xNode     *node;
	      
	      // a xAddition is created to be returned
	      addition= new xAddition($2,$4);

	      Free_Nodes+=addition;

	      node=$2;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      node=$4;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      addition->Alpha=Command_Counter;
	      addition->Beta=Command_Counter;

	      $$=addition;

	      // cout <<"+expression parsed!!!\n";

	      // this rule raises only normal syntax-error!!!
	    }
	  | '(' expression '-' expression ')'
	    {
	      TIndex       index;
	      xSubtraction *subtraction;
	      xNode        *node;

	      // a xSubtraction is created to be returned
	      subtraction= new xSubtraction($2,$4);

	      Free_Nodes+=subtraction;

	      node=$2;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      node=$4;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      subtraction->Alpha=Command_Counter;
	      subtraction->Beta=Command_Counter;

	      $$=subtraction;

	      // cout <<"-expression parsed!!!\n";

	      // this rule raises only normal syntax-error!!!
	    }
	  | '(' expression '*' expression ')'
	    {
	      TIndex          index;
	      xMultiplication *multiplication;
	      xNode           *node;

	      // a xMultiplication is created to be returned
	      multiplication= new xMultiplication($2,$4);

	      Free_Nodes+=multiplication;

	      node=$2;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      node=$4;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      multiplication->Alpha=Command_Counter;
	      multiplication->Beta=Command_Counter;

	      $$=multiplication;

	      // cout <<"*expression parsed!!!\n";

	      // this rule raises only normal syntax-error!!!
	    }
	  | '(' expression '/' expression ')'
	    {
	      TIndex    index;
	      xDivision *division;
	      xNode     *node;
	      
	      // a xDivision is created to be returned
	      division= new xDivision($2,$4);

	      Free_Nodes+=division;

	      node=$2;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      node=$4;

	      index=Free_Nodes(node);
	      Free_Nodes-=index;

	      division->Alpha=Command_Counter;
	      division->Beta=Command_Counter;

	      $$=division;

	      // cout <<":expression parsed!!!\n";

	      // this rule raises only normal syntax-error!!!
	    }
	  ;
	    
	    
array: IDENTIFIER '[' expression ']'
       {
	 TIndex   index;
	 xString  symbol;
	 xArray   *array;
	 xNode    *node;

	 // a xString is created out of the IDENTIFIER
	 symbol=xString($1);

	 // a xArray is created
	 array=new xArray(symbol, $3);

	 Free_Nodes+=array;

	 node=$3;

	 index=Free_Nodes(node);
	 Free_Nodes-=index;

	 // checking if the array has been used as a variable before
	 if (Variable_Symboltable.Contains(array))
	   {
	     yyerror("Array used as a variable before!!!");
	     return 1;
	   }

	 // checking if the array has been used as a function before
	 if (Local_Functions_Symboltable.Contains(array))
	   {
	     yyerror("Array used as a function before!!!");
	   }
     
	 array->Alpha=Command_Counter;
	 array->Beta=Command_Counter;
	 
	 // the array is inserted into the Array_Symboltable
	 Array_Symboltable+=array;

      	 $$=array;

	 // cout <<"array-parsed!!!\n";

	 // this rule raises an error if the array has been used 
	 // as a variable or a function before
       }
     ;


condition: '(' expression '<' '=' expression ')'
	   {
	     TIndex           index;
	     xSmallerEqual    *bool_expression;
	     xNode            *node;

	     // a xSmallerEqual is created to be returned
	     bool_expression=new xSmallerEqual($2,$5);

	     Free_Nodes+=bool_expression;

	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$5;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<"<=condition parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 | '(' expression '>' '=' expression ')'
	   {
	     TIndex        index;
	     xGreaterEqual *bool_expression;
	     xNode         *node;
	       
	     // an xGreaterEqual is created to be returned
	     bool_expression=new xGreaterEqual($2,$5);

	     Free_Nodes+=bool_expression;


	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$5;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<">=condition parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 | '(' expression '<' expression ')'
	   {
	     TIndex   index;
	     xSmaller *bool_expression;
	     xNode    *node;

	     // a xSmaller is created to be returned
	     bool_expression= new xSmaller($2,$4);

	     Free_Nodes+=bool_expression;

	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;


	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<"<condition parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 | '(' expression '>' expression ')'
	   {
	     TIndex   index;
	     xGreater *bool_expression;
	     xNode    *node;

	     // a xGreater is created to be returned
	     bool_expression= new xGreater($2,$4);

	     Free_Nodes+=bool_expression;


	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;
	     
	     // cout <<">condition parsed!!!\n"; 

	     // this rule raises only normal syntax-error!!!
	   }
	 | '(' expression '=' expression ')'
	   {
	     TIndex index;
	     xEqual *bool_expression;
	     xNode  *node;

	     // a xEqual is created to be returned
	     bool_expression= new xEqual($2,$4);

	     Free_Nodes+=bool_expression;

	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;	     

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<"=condition parsed!!!\n";
	   }
	 | '(' expression '#' expression ')'
	   {
	     TIndex   index;
	     xUnEqual *bool_expression;
	     xNode    *node;

	     // a xUnEqual is created
	     bool_expression= new xUnEqual($2,$4);

	     Free_Nodes+=bool_expression;

	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<"#condition parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 | NOT '(' condition ')'
	   {
	     TIndex index;
	     xNot   *bool_expression;
	     xNode  *node;

	     // a xNot is created to be returned
	     bool_expression= new xNot($3);

	     Free_Nodes+=bool_expression;

	     node=$3;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<"NOT-condition parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 | '(' condition AND condition ')'
  	   {
	     TIndex  index;
	     xAnd    *bool_expression;
	     xNode   *node;

	     // a xAnd is created to be returned
	     bool_expression= new xAnd($2,$4);

	     Free_Nodes+=bool_expression;

	     
	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;


	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;

	     // cout <<"AND-condition parsed!!!\n";

	     // this rule raises only normal syntax-error!!!
	   }
	 | '(' condition OR condition ')' 
	   {
	     TIndex  index;
	     xOr     *bool_expression;
	     xNode   *node;

	     // a xOr is created to be returned
	     bool_expression=new xOr($2,$4);

	     Free_Nodes+=bool_expression;

	     node=$2;

	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     node=$4;
	     
	     index=Free_Nodes(node);
	     Free_Nodes-=index;

	     bool_expression->Alpha=Command_Counter;
	     bool_expression->Beta=Command_Counter;

	     $$=bool_expression;
	     
	     // cout <<"OR-condition parsed!!!\n";
	     // this rule raises only normal syntax-error!!!
	   }
	 ;

%%




extern FILE *yyin;
FILE *file;

 
int main(int argc, char **argv)
{
  char *Outfile;
  xString File;

  xPascalliTranslator *backend;

  try
  {
    if (argc == 3)
    {
      Outfile = argv[2];
      file = fopen(argv[1], "r");
      if (file)
      {
	yyin = file;
	ungetc('\n',file);	
      }
      else
      {
	cout << "Could not open file: " << argv[1] << "\n";
	return 1;
      };

    }
    else
    {
      cout << "syntax: plic <infile> <outfile>!\n";
      return 1;
    };

    do
    {
      Status = yyparse();
    }
    while (!feof(yyin));

    File = xString(Outfile);

    if (Status == 0)
    {
      // the following rutine checks if every used function is also
      // delcared and raises an error if this is not the case
    
      if (used_but_not_declared())
      {
	//free_parstree(); weil segfault FIXME Postsegfault bug. (erikyyy)

	message_abnormal_compilation();
	return 1;
      };

      // the Function_Symboltables are reseted
      Used_Functions_Symboltable = New_Symboltable;
      Declared_Functions_Symboltable = New_Symboltable;

      // xPascalliTranslator is called
      backend = new xPascalliTranslator(TopNode, File); // corrected
    }
    else
    {
      message_abnormal_compilation();
      //free_parstree(); weil segfault FIXME
      return 1;
    }
  }
  catch(xBaseException & x)
  {
    char msg[10000];		// fixme design error in xbaseexception

    x.GetDescription(msg);
    cout << msg << "\n\n\n";
    message_abnormal_compilation();
    return 1;
  }
  catch(...)
  {
    cout << "unknnown exception caught\n\n\n\n";
    message_abnormal_compilation();
    return 1;
  };
  message_compilation_finished();
  //free_parstree();FIXME
  return 0;
};




void free_parstree()
{
  // cout << "Freeing parse-tree..." <<endl;

  // the parse-tree is freed

  // delete(TopNode);

  for (TIndex i=0 ; i<~Free_Nodes; i++)
    {
      delete(Free_Nodes[i]);
    }
  
  // cout <<"OK!" <<endl;

  // cout << "Freeing identifiers..." <<endl;
  
  for (TIndex i=0 ; i<~Free_Identifiers; i++)
    {
      delete(Free_Identifiers[i]);
    }
  
  // cout <<"OK!" <<endl;
  
  
  // cout <<"Freeing Arglists..." <<endl;
  
  for (TIndex i=0 ; i<~Free_IdentifierLists; i++)
    {
      delete( Free_IdentifierLists[i]);
    }
  
  // cout <<"OK" <<endl;
  
  
  // cout <<"Freeing Arglists (second part) " <<endl;
  
  for (TIndex i=0 ; i<~Free_ExpressionLists ; i++)
    {
      delete(Free_ExpressionLists[i]);
    }
  
  // cout <<"OK" <<endl;
  
  // cout <<"Freeing Functionlists..." <<endl;
  
  for (TIndex i=0 ; i<~Free_FunctionLists ; i++)
    {
      delete(Free_FunctionLists[i]);
    }
  
  // cout <<"OK" <<endl;
  
  
  // cout <<"Freeing Commandlists..." <<endl;
  
  for (TIndex i=0 ; i<~Free_CommandLists ; i++)
    {
      delete(Free_CommandLists[i]);
    }
  
  // cout <<"OK" <<endl;
  
}


TBool used_but_not_declared()
{
  TIndex      index;
  xIdentifier *declared_function;
 
  for (TIndex i=0 ; i<~Used_Functions_Symboltable ; i++)
    {
      if (!(Declared_Functions_Symboltable.Contains(Used_Functions_Symboltable[i])))
	{
	  cout << "Error: Function " << Used_Functions_Symboltable[i]->Name <<" used but not declared!!!" << endl;
	  return TRUE;
	}
      else	
	{
	  // getting the function
	  index=Declared_Functions_Symboltable+=Used_Functions_Symboltable[i];
	  declared_function=Declared_Functions_Symboltable[index];
	  if ((declared_function->Number_Of_Arguments) != (Used_Functions_Symboltable[i]->Number_Of_Arguments))
	    {
	      cout <<"Number of parameters in call to function " <<Used_Functions_Symboltable[i]->Name << " does not match declaration!!!" << endl;
		return TRUE;		
	    }
	  
	}
    }
  return FALSE;
}

void message_abnormal_compilation()
{
  struct tm *pTime;
  time_t t;
  
  t=time(NULL);
  pTime=localtime(&t);
  
  cout << "Compilation exited abnormally at " << asctime(pTime);
}

void message_compilation_finished()
{
struct tm *pTime;
time_t t;

t=time(NULL);
pTime=localtime(&t);

 cout << "Compilation finished at " << asctime(pTime);
}
