/*
  plic visitor
  Copyright (C) 1999  Achim Kern

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ---------------------------------------------------------------------------- 
//  Description      : Implementation of the ancestor of all passes forming the
//                     backend of the pascalli to pascallino compiler.
//                     
//                     This visitor proivides function to travers a pascalli-
//                     parsetree and just rebuild the original program.
// ---------------------------------------------------------------------------- 
//  Creation         :
// ---------------------------------------------------------------------------- 
//  Remarks          : 
// ---------------------------------------------------------------------------- 
//  Modifications    : 
//      $Log: pli_visitor.cpp,v $
// Revision 1.2  1999/07/19  14:50:45  thieleek
// first complete revision
//
// Revision 1.1.1.1  1999/07/13  13:35:50  thieleek
// sources Imported
//
// ---------------------------------------------------------------------------- 




#include "plic_Copier.h"




xPascalliCopier::xPascalliCopier()
{ 
};




xNode* xPascalliCopier::
startCompilation(xNode* topnode)
{
  topnode->AcceptVisitor(*this);
  return Results.pop();
};





void xPascalliCopier::
visit_xProgramNode(xProgramNode & node)
{
  xProgramNode* NewNode = 0;

  NewNode = new xProgramNode(*(new xFunctionList));
  for (TIndex Counter = 0; Counter < ~(node.Functions); Counter++)
  {
    node.Functions[Counter]->AcceptVisitor(*this);
    NewNode->Functions += (xFunction*)(Results.pop());
  };
  Results.push(NewNode);
};




void xPascalliCopier::
visit_xFunction(xFunction & node)
{
  xFunction* NewNode = 0;

  NewNode = new xFunction(new xIdentifier(node.Name.Name), *(new xIdentifierList), 0);
  for (TIndex Counter = 0; Counter < ~(node.Arguments); Counter++)
  {
    node.Arguments[Counter]->AcceptVisitor(*this);
    NewNode->Arguments += (xIdentifier*)(Results.pop());
  };
  node.Command->AcceptVisitor(*this);
  NewNode->Command = (xCommand*)(Results.pop());

  Results.push(NewNode);
};




void xPascalliCopier::
visit_xCommandBlock(xCommandBlock & node)
{
  xCommandBlock* CurrentCommandBlock;

  InsertationPoint.push(new xCommandBlock);
  for (TIndex Counter = 0; Counter < ~(node.CommandList); Counter++)
  {
    node.CommandList[Counter]->AcceptVisitor(*this);
    CurrentCommandBlock = InsertationPoint.pop();
    CurrentCommandBlock->addCommand((xCommand*)(Results.pop()));
    InsertationPoint.push(CurrentCommandBlock);
  };
  Results.push(InsertationPoint.pop());
};




void xPascalliCopier::
visit_xAssignment(xAssignment & node)
{
  xIdentifier* Dest   = 0;
  xExpression* Source = 0;

  node.Destination->AcceptVisitor(*this);  
  node.Source->AcceptVisitor(*this);
  Source = (xExpression*)(Results.pop());
  Dest = (xIdentifier*)(Results.pop());
   
  Results.push(new xAssignment(Dest, Source));
};




void xPascalliCopier::
visit_xWhile(xWhile & node)
{
  xCommand*   Command = 0;
  xCondition* Condition = 0;

  node.Command->AcceptVisitor(*this);
  node.Condition->AcceptVisitor(*this);
  Condition = (xCondition*)(Results.pop());
  Command = (xCommand*)(Results.pop());

  Results.push(new xWhile(Condition, Command));
};




void xPascalliCopier::
visit_xIfStatement(xIfStatement & node)
{
  xIfStatement* NewNode = 0;
  xCondition* Condition = 0;
  xCommand* Command = 0;

  node.IfTwig->AcceptVisitor(*this);
  node.Condition->AcceptVisitor(*this);
  Condition = (xCondition*)(Results.pop());
  Command =  (xCommand*)(Results.pop());

  Results.push(new xIfStatement(Condition, Command, 0));
  if (node.ElseTwig != 0)
  {
    NewNode = (xIfStatement*)(Results.pop());
    node.ElseTwig->AcceptVisitor(*this);
    NewNode->ElseTwig = (xCommand*)(Results.pop());
    Results.push(NewNode);
  };
};





void xPascalliCopier::
visit_xForLoop(xForLoop & node)
{
  node.Command->AcceptVisitor(*this);
  node.EndValue->AcceptVisitor(*this);
  node.StartValue->AcceptVisitor(*this);
  node.Counter->AcceptVisitor(*this);

  Results.push(new xForLoop((xIdentifier*)(Results.pop()), (xExpression*)(Results.pop()), (xExpression*)(Results.pop()), (xCommand*)(Results.pop())));
};




void xPascalliCopier::
visit_xAddition(xAddition & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());
 
  Results.push(new xAddition(Left, Right));
};




void xPascalliCopier::
visit_xSubtraction(xSubtraction & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());

  Results.push(new xSubtraction(Left, Right));
};




void xPascalliCopier::
visit_xMultiplication(xMultiplication & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());

  Results.push(new xMultiplication(Left, Right));
};




void xPascalliCopier::
visit_xDivision(xDivision & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());

  Results.push(new xDivision(Left, Right));
};




void xPascalliCopier::
visit_xConstant(xConstant & node)
{
  Results.push(new xConstant(node.Value));
};




void xPascalliCopier::
visit_xIdentifier(xIdentifier & node)
{
  Results.push(new xIdentifier(node.Name));
};




void xPascalliCopier::
visit_xArray(xArray & node)
{
  node.Index->AcceptVisitor(*this);
  
  Results.push(new xArray(node.Name, (xExpression*)(Results.pop())));
};




void xPascalliCopier::
visit_xFunctionCall(xFunctionCall & node)
{
  xFunctionCall* NewNode = 0;

  NewNode = new xFunctionCall(node.Name, *(new xExpressionList));
  for (TIndex Counter = 0; Counter < ~(node.Arguments); Counter++)
  {
    node.Arguments[Counter]->AcceptVisitor(*this);
    NewNode->Arguments += (xExpression*)(Results.pop());
  };

  Results.push(NewNode);
};




void xPascalliCopier::
visit_xAnd(xAnd & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xCondition* Right = (xCondition*)(Results.pop());
  xCondition* Left = (xCondition*)(Results.pop());

  Results.push(new xAnd(Left, Right));
};




void xPascalliCopier::
visit_xOr(xOr & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xCondition* Right = (xCondition*)(Results.pop());
  xCondition* Left = (xCondition*)(Results.pop());
  
  Results.push(new xOr(Left, Right));
}




void xPascalliCopier::
visit_xNot(xNot & node)
{
  node.Expression->AcceptVisitor(*this);
  
  Results.push(new xNot((xCondition*)(Results.pop())));
};




void xPascalliCopier::
visit_xGreaterEqual(xGreaterEqual & node)
{  
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());

  Results.push(new xGreaterEqual(Left, Right));
};




void xPascalliCopier::
visit_xGreater(xGreater & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());
 
  Results.push(new xGreater(Left, Right));
};




void xPascalliCopier::
visit_xEqual(xEqual & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());
 
  Results.push(new xEqual(Left, Right));
};




void xPascalliCopier::
visit_xSmaller(xSmaller & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());

  Results.push(new xSmaller(Left, Right));
};




void xPascalliCopier::
visit_xSmallerEqual(xSmallerEqual & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());
 
  Results.push(new xSmallerEqual(Left, Right));
};




void xPascalliCopier::
visit_xUnEqual(xUnEqual & node)
{
  node.Left->AcceptVisitor(*this); 
  node.Right->AcceptVisitor(*this);
  xExpression* Right = (xExpression*)(Results.pop());
  xExpression* Left = (xExpression*)(Results.pop());
 
  Results.push(new xUnEqual(Left, Right));
};





















