/*
  plic visitor
  Copyright (C) 1999  Achim Kern

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ---------------------------------------------------------------------------- 
//  Description      : Definition of xPascalliVistor.
//                     xPascalliVisitor is a child of xVisitor
//                     it is an abstact object only used to define a common 
//                     interface to all passes of the several passes of a 
//                     Pascalli compiler
// ---------------------------------------------------------------------------- 
//  Creation         : 05.07.1999
// ---------------------------------------------------------------------------- 
//  Remarks          : 
// ---------------------------------------------------------------------------- 
//  Modifications    : 
//      $Log: 
//      
// ---------------------------------------------------------------------------- 




#ifndef PLI_TRAVERSER_H
#define PLI_TRAVERSER_H




#include "commontype.h"
#include "frontend.h"
#include "backend.h"
#include "Stack.h"




typedef xStack<xNode*> xNodeStack;
typedef xStack<xCommandBlock*> xCommandBlockStack;




class xPascalliTraverser : public xPascalliVisitor
{
 public:
  xPascalliTraverser();
  virtual ~xPascalliTraverser(){};
  
  xNode* startCompilation(xNode * topnode);
  virtual void visit_xProgramNode(xProgramNode & node);
  virtual void visit_xFunction(xFunction & node);
  virtual void visit_xCommandBlock(xCommandBlock & node);
  virtual void visit_xAssignment(xAssignment & node);
  virtual void visit_xWhile(xWhile & node);
  virtual void visit_xIfStatement(xIfStatement & node);
  virtual void visit_xForLoop(xForLoop & node);
  virtual void visit_xAddition(xAddition & node);
  virtual void visit_xSubtraction(xSubtraction & node);
  virtual void visit_xMultiplication(xMultiplication & node);
  virtual void visit_xDivision(xDivision & node);
  virtual void visit_xConstant(xConstant & node);
  virtual void visit_xIdentifier(xIdentifier & node);
  virtual void visit_xArray(xArray & node);
  virtual void visit_xFunctionCall(xFunctionCall & node);
  virtual void visit_xAnd(xAnd & node);
  virtual void visit_xOr(xOr & node);
  virtual void visit_xNot(xNot & node);
  virtual void visit_xGreaterEqual(xGreaterEqual & node);
  virtual void visit_xGreater(xGreater & node);
  virtual void visit_xEqual(xEqual & node);
  virtual void visit_xSmaller(xSmaller & node);
  virtual void visit_xSmallerEqual(xSmallerEqual & node);
  virtual void visit_xUnEqual(xUnEqual & node);
 protected:
  xNodeStack          Results;
  xCommandBlockStack  InsertationPoint;
 private:
};




#endif
