/*
  plic visitor
  Copyright (C) 1999  Achim Kern

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ---------------------------------------------------------------------------- 
//  Description      : Implementation of the ancestor of all passes forming the
//                     backend of the pascalli to pascallino compiler.
//                     
//                     This visitor proivides function to travers a pascalli-
//                     parsetree and just rebuild the original program.
// ---------------------------------------------------------------------------- 
//  Creation         :
// ---------------------------------------------------------------------------- 
//  Remarks          : 
// ---------------------------------------------------------------------------- 
//  Modifications    : 
//      $Log: pli_visitor.cpp,v $
// Revision 1.2  1999/07/19  14:50:45  thieleek
// first complete revision
//
// Revision 1.1.1.1  1999/07/13  13:35:50  thieleek
// sources Imported
//
// ---------------------------------------------------------------------------- 




#include "plic_Writer.h"
#include <iostream>
#include <fstream>



xPascalliWriter::xPascalliWriter()
{
};




void xPascalliWriter::
visit_xProgramNode(xProgramNode & node)
{
  for (TIndex Counter = 0; Counter < ~(node.Functions); Counter++)
  {
    node.Functions[Counter]->AcceptVisitor(*this);
  };
};




void xPascalliWriter::
visit_xFunction(xFunction & node)
{
  for (TIndex Counter = 0; Counter < ~(node.Arguments); Counter++)
  {
    node.Arguments[Counter]->ResultAddress = node.Arguments[Counter]->Name;
  };
  Program.add("function " + node.Name.Name + Program.get_Argumentlist(node.Arguments, ~(node.Arguments)) + ";");
  Program.add_Begin();

  node.Command->AcceptVisitor(*this);

  Program.add_EndofFunction();
};




void xPascalliWriter::
visit_xCommandBlock(xCommandBlock & node)
{
  for (TIndex Counter = 0; Counter < ~(node.CommandList); Counter++)
  {
    node.CommandList[Counter]->AcceptVisitor(*this);
  };
};




void xPascalliWriter::
visit_xAssignment(xAssignment & node)
{
  node.Destination->AcceptVisitor(*this);
  node.Source->AcceptVisitor(*this);

  Program.add_Assign(node.Destination->ResultAddress, node.Source->ResultAddress);
};




void xPascalliWriter::
visit_xWhile(xWhile & node)
{
  node.Condition->AcceptVisitor(*this);

  Program.add_While(node.Condition->ResultAddress);

  node.Command->AcceptVisitor(*this);

  Program.add_End();
};




void xPascalliWriter::
visit_xIfStatement(xIfStatement & node)
{
  node.Condition->AcceptVisitor(*this);

  Program.add_If(node.Condition->ResultAddress);

  node.IfTwig->AcceptVisitor(*this);

  Program.add_End();

  if (node.ElseTwig != 0)
  {
    Program.add("else");
    Program.add_Begin();

    node.ElseTwig->AcceptVisitor(*this);

    Program.add_End();
  };
};





void xPascalliWriter::
visit_xForLoop(xForLoop & node)
{
  node.Counter->AcceptVisitor(*this);
  node.StartValue->AcceptVisitor(*this);
  node.EndValue->AcceptVisitor(*this);

  Program.add_For(node.Counter->ResultAddress, node.StartValue->ResultAddress, node.EndValue->ResultAddress);

  node.Command->AcceptVisitor(*this);

  Program.add_End();

};




void xPascalliWriter::
visit_xAddition(xAddition & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + cat(node.Left->ResultAddress, "+", node.Right->ResultAddress) + ")";

};




void xPascalliWriter::
visit_xSubtraction(xSubtraction & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + cat(node.Left->ResultAddress, "-", node.Right->ResultAddress) + ")";
};




void xPascalliWriter::
visit_xMultiplication(xMultiplication & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);
  
  node.ResultAddress = "(" + cat(node.Left->ResultAddress, "*", node.Right->ResultAddress) + ")";
};




void xPascalliWriter::
visit_xDivision(xDivision & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + cat(node.Left->ResultAddress, "/", node.Right->ResultAddress) + ")";
};




void xPascalliWriter::
visit_xConstant(xConstant & node)
{
  node.ResultAddress = node.Value;
};




void xPascalliWriter::
visit_xIdentifier(xIdentifier & node)
{
  node.ResultAddress = node.Name;
};




void xPascalliWriter::
visit_xArray(xArray & node)
{
  node.Index->AcceptVisitor(*this);
  node.ResultAddress = node.Name + "[" + node.Index->ResultAddress + "]";
};




void xPascalliWriter::
visit_xFunctionCall(xFunctionCall & node)
{
  for (TIndex Counter = 0; Counter < ~(node.Arguments); Counter++)
  {
    node.Arguments[Counter]->AcceptVisitor(*this);
  };
  node.ResultAddress = node.Name + Program.get_Argumentlist(node.Arguments, ~(node.Arguments));
};




void xPascalliWriter::
visit_xAnd(xAnd & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " and " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xOr(xOr & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " or " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xNot(xNot & node)
{
  node.Expression->AcceptVisitor(*this);

  node.ResultAddress = "not (" + node.Expression->ResultAddress + ")";
};




void xPascalliWriter::
visit_xGreaterEqual(xGreaterEqual & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " >= " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xGreater(xGreater & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " > " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xEqual(xEqual & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " = " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xSmaller(xSmaller & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " < " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xSmallerEqual(xSmallerEqual & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " <= " + node.Right->ResultAddress + ")";
};




void xPascalliWriter::
visit_xUnEqual(xUnEqual & node)
{
  node.Left->AcceptVisitor(*this);
  node.Right->AcceptVisitor(*this);

  node.ResultAddress = "(" + node.Left->ResultAddress + " # " + node.Right->ResultAddress + ")";
};




xString & xPascalliWriter::
cat(xString str1, xString str2, xString str3)
{
  xString *tmp;

  tmp = new xString;
  *tmp = str1 + " " + str2 + " " + str3;
  return *tmp;
};




xString & xPascalliWriter::
cat(xString str1, xString str2)
{
  xString *tmp;

  tmp = new xString;
  *tmp = str1 + " " + str2;
  return *tmp;
};




void xPascalliWriter::
startWriting(xNode * topnode, xString filename)
{
  Program.Empty();
  //  cout << "creating program..." << endl;
  topnode->AcceptVisitor(*this);
  ofstream OutPutFile(filename);
  OutPutFile << Program;
  //  cout << ".. [ok]" << endl;
};
