/*
  modul for xVisitor
  Copyright (C) 1999  Achim Kern

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ----------------------------------------------------------------------------
//  Description      : modul for xVisitor
//
//  To Do: xArglist
// ----------------------------------------------------------------------------




#include "backend.h"
#include <fstream.h>
#include <iostream.h>




xString xPascallinoVisitor::Index2Register(TIndex regno)
{
  xString str;
  str = "R" + xString::Unsigned2Dec(regno);
  return str;
}




// streichen
xString xPascallinoVisitor::InsertCCL()
{
  return xString::Unsigned2Dec(CCL) + " ";
}  




xPascallinoVisitor::xPascallinoVisitor(TIndex no_of_args, TIndex no_of_var, TIndex no_of_array, xFunction *main_function, xString filename)
{
  // do initalization stuff
  MemManager.init(no_of_var, no_of_array, (no_of_args + 5));
  FileName = filename;
  Zero = Index2Register(no_of_args + 1);
  Program += Zero + " <- 0";
  One = Index2Register(no_of_args + 2);
  Program += One + " <- 1";
  PageSize = Index2Register(no_of_args + 3);
  Program +=  PageSize + " <- " + xString::Unsigned2Dec(no_of_array + 1);
  Counter = Index2Register(no_of_args + 4);
  // start the traversation of the parse-tree
  main_function->AcceptVisitor(*this);
}




void xPascallinoVisitor::visit_xFunction(xFunction *Node)
{
  // translate ArgList of the function
  Node->ArgList->AcceptVisitor(*this);
  // traverse the command of the function
  Node->Command->AcceptVisitor(*this); 
  // assign the output to Register R0
  Program += "R0 <- " + Index2Register(MemManager.GetAddress(*(Node->Name)));
  Program += "STOP";
  //  cout << "File wird generiert" << endl;
  ofstream stream(FileName);
  stream << Program;
};




void xPascallinoVisitor::visit_xParameter(xParameter *Node)
{
  for (TIndex index = 0; index < ~(*Node->ArgList); index++)
  {
    xIdentifier *var = (*Node->ArgList)[index];
    var->Address->Address = new TIndex; 
    *(var->Address->Address) = index;
  }
}




void xPascallinoVisitor::visit_xCommandBlock(xCommandBlock *Node)
{
  TSize Index;
  // visit each entry of commandlist and translate it
  for (Index = 0; Index < ~(*Node->CommandList); Index++)
  {
    (*Node->CommandList)[Index]->AcceptVisitor(*this);
  }
};




void xPascallinoVisitor::visit_xCondition(xCondition *Node)
{
  // As we know that in pascallino a condition can only be Identifier <> 0 we can make translation pretty easy.
  // we just traverse the expression, which must be an Identifier and than copy the value
  // of Identifier.ReturnAddres into Condition.ReturnAddress
  Node->Expression->AcceptVisitor(*this);
  Node->ResultAddress = Node->Expression->ResultAddress;
};



void xPascallinoVisitor::visit_xWhile(xWhile *Node)
{
  TIndex StartAddress;
  xString tmp;
  
  Node->Condition->AcceptVisitor(*this);  
  Program +=   "IF "+ Node->Condition->ResultAddress + " = 0 GOTO ";
  StartAddress = ~Program;  // save the current outputline for being able to jump back later
  Node->Command->AcceptVisitor(*this);
  Program +=  "GOTO " + xString::Unsigned2Dec(StartAddress);
  Program[StartAddress - 1] += xString::Unsigned2Dec(~Program + 1);
};




void xPascallinoVisitor::visit_xAssignment(xAssignment *Node)
{
  Node->Destination->AcceptVisitor(*this);
  Node->Source->AcceptVisitor(*this);
  
  Program +=  Node->Destination->ResultAddress + " <- " + Node->Source->ResultAddress;
};




void xPascallinoVisitor::visit_xConstant(xConstant *Node)
{
  Node->ResultAddress = Node->Value;
}




void xPascallinoVisitor::visit_xSum(xSum *Node)
{
  xString TmpReg;

  Node->Left->AcceptVisitor(*this);
  Node->Right->AcceptVisitor(*this);
  // now add the to values standing in the resultregister of the both expressions
  TmpReg = "R" + xString::Unsigned2Dec(MemManager.GetTmpReg());
  Program +=  TmpReg + " <- " + Node->Left->ResultAddress + " + " + Node-> Right->ResultAddress;
  // assign the DestinationAddress to the field ResultAddress and exit 
  Node->ResultAddress = TmpReg;
};




void xPascallinoVisitor::visit_xDiff(xDiff *Node)
{
  xString TmpReg;

  Node->Left->AcceptVisitor(*this);
  Node->Right->AcceptVisitor(*this);
  // now add the to values standing in the resultregister of the both expressions
  TmpReg = "R" + xString::Unsigned2Dec(MemManager.GetTmpReg());
  Program +=  TmpReg + " <- " + Node->Left->ResultAddress + " - " + Node-> Right->ResultAddress;
  // assign the DestinationAddress to the field ResultAddress and exit 
  Node->ResultAddress = TmpReg;
};




void xPascallinoVisitor::visit_xArray(xArray *Node)
{
  xString Index;           // this is the name of the Register in which the index of the array will be calculated
  xString TmpReg;
  TIndex  BaseAddress;
    
  Index = Index2Register(MemManager.GetTmpReg());
  TmpReg = Index2Register(MemManager.GetTmpReg());
  // first travers the expressionnode
  Node->Index->AcceptVisitor(*this);
  // get the baseaddress of the array
  BaseAddress = MemManager.GetAddress(*Node);
  Program += Index + " <- " + xString::Unsigned2Dec(BaseAddress);
  Program += Counter + " <- " + Node->Index->ResultAddress;
  Program += "IF " + Counter + " = 0 GOTO " + xString::Unsigned2Dec(~Program + 5);
  Program += Index + " <- " + Index + " + " + PageSize;
  Program += Counter + " <- " + Counter + " - " + One;
  Program += "GOTO " + xString::Unsigned2Dec(~Program - 2);
  //  Program += TmpReg + " <- " + Index;  
  Node->ResultAddress = "R" + Index;
}; 




void xPascallinoVisitor::visit_xVariable(xVariable *Node)
{
  Node->ResultAddress = Index2Register(MemManager.GetAddress(*Node));  
};



































