/*
  modul for xVisitor
  Copyright (C) 1999  Achim Kern

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "commontype.h"
#include "xVisitor.h"
#include "prgram.h"
#include "frontend.h"




typedef xList<TIndex> xRegList;




class xMemoryManager
{
 public:
  void Free(TIndex RegisterNo);       
  TIndex GetAddress(xIdentifier& Node);
  TIndex GetTmpReg();
  xMemoryManager();
  void init(TIndex noofvar, TIndex noofarray, TIndex baseaddress);
 private:
  xRegList    FreedTmpReg;
  TIndex      LastUsedTmpRegister;
  TIndex      LastUsedArrayRegister;
  TIndex      LastUsedVarRegister;
  TIndex      PageSize;
};




class xPascallinoVisitor : xVisitor<xRAMProgram, xMemoryManager>
{ 
 public:
  xPascallinoVisitor(TIndex no_of_args, TIndex no_of_var, TIndex no_of_array, xFunction *main_function, xString filename); 
  virtual ~xPascallinoVisitor(){};

  virtual void visit_xFunction(xFunction *Node);
  virtual void visit_xParameter(xParameter *Node);
  virtual void visit_xCommandBlock(xCommandBlock *Node);
  virtual void visit_xWhile(xWhile *Node);
  virtual void visit_xCondition(xCondition *Node);
  virtual void visit_xAssignment(xAssignment *Node);
  virtual void visit_xExpression(xExpression *Node){};
  virtual void visit_xVariable(xVariable *Node);
  virtual void visit_xArray(xArray *Node);
  virtual void visit_xConstant(xConstant *Node);
  virtual void visit_xSum(xSum *Node);
  virtual void visit_xDiff(xDiff *Node);

  virtual void visit_xNode(struct xNode *){};
  virtual void visit_xCommand(struct xCommand *){};
  virtual void visit_xIfStatement(struct xIfStatement *){};
  virtual void visit_xForLoop(struct xForLoop *){};
  virtual void visit_xAddition(struct xAddition *){};
  virtual void visit_xSubtraction(struct xSubtraction *){};
  virtual void visit_xMultiplication(struct xMultiplication *){};
  virtual void visit_xDivision(struct xDivision *){};
  virtual void visit_xIdentifier(struct xIdentifier *){};
  virtual void visit_xCall(struct xFunctionCall *){};
  virtual void visit_xBooleshExpression(struct xBoolshExpression *){};
  virtual void visit_xAnd(struct xAnd *){};
  virtual void visit_xOr(struct xOr *){};
  virtual void visit_xNot(struct xNot *){};
  virtual void visit_xGreaterEqual(struct xGreaterEqual *){};
  virtual void visit_xGreater(struct xGreater *){};
  virtual void visit_xEqual(struct xEqual *){};
  virtual void visit_xSmaller(struct xSmaller *){};
  virtual void visit_xSmallerEqual(struct xSmallerEqual *){};
  virtual void visit_xUnEqual(struct xUnEqual *){};

 protected:
 
 private:
  // xRAMProgram     DestProg;              // this is the destination programm
  TIndex          CCL;                   // this variable is to count the current line in the destination program DestProg,
                                         // the line where to insert new code
  xString         Zero;                  // this variable hold the name of the register loaded with 0. This register is used if
                                         // the compiler neads to calculate something. e.g. array-indexes
  xString         One;                   // look at Zero. In contrast to Zero this Register is loaded with 1
  xString         PageSize;               
  xString         Counter;               // this Register is used when the compiler nead a loopcounter. e.g. array-indexes
  //  xMemoryManager  MemManager;            // an object which manages memoryallocation
  xString         FileName;
			  
  xString Index2Register(TIndex regno); // this methode converts a TIndex an an xString of the form "R" + TIndex
  xString InsertCCL();                  // this mehtode returns CCL 
}; 




