/*
  Implementation of the parse tree
  Copyright (C) 1999  Philipp Cimiano Lavin

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ---------------------------------------------------------------------
// Description   : Implementation of the parse tree
// ---------------------------------------------------------------------
// Remarks       :
//        Philipp 06.02.1999
// ---------------------------------------------------------------------
// Modifications :
//        Philipp 06.02.1999  created.
// ---------------------------------------------------------------------




#include "commontype.h"
#include "backend.h"
#include "frontend.h"
#include <iostream.h>




xNode::xNode()
  // :ResultAddress(0)
{
}




xNode::~xNode()
{
}




xFunction::xFunction(xVariable  *name,
		     xParameter *arglist,
		     xCommand   *command)
: Name(name),ArgList(arglist),Command(command)
{
}




xFunction::~xFunction()
{
  if (ArgList)
  {
    delete(ArgList);
    ArgList = 0;
  }
}




void xFunction::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xFunction(this);
}




void xFunction::print()
{
  cout << "xFunction_Node: ";
  if (Name)
    {
      Name->print();
    }
  if (ArgList)
    {
    ArgList->print();
    }
  if (Command)
    {
      Command->print();
    }
  cout <<"\n";
}




xParameter::xParameter(xArgList *arglist)
:ArgList(arglist)
{
}




xParameter::~xParameter()
{
  if (ArgList) 
  {
    delete (ArgList);
    ArgList = 0;
  }
}




void xParameter::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xParameter(this);
}




void xParameter::print()
{
  cout << "\n\txParameter_Node:";
  if (ArgList)
    {
      for (TIndex index=0; index<~(*ArgList); index++)
	{
	  cout <<"\n\t";
	  ((*ArgList)[index])->print();
	}
   }
   cout <<"\n";
}




xCondition::xCondition(xExpression *expression)
: Expression(expression)
{
}




xCondition::~xCondition()
{
}




void xCondition::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xCondition(this);
}




void xCondition::print()
{
  cout <<"xCondition_Node: ";
  if (Expression)
    {
      cout <<"(";
      Expression->print();
      cout << " # 0)"; 
    }
}




xCommandBlock::xCommandBlock(xCommandList *commandlist)
: CommandList(commandlist)
{
}




xCommandBlock::~xCommandBlock()
{
  if (CommandList) 
  {
    delete (CommandList);
    CommandList = 0;
  }
}




void xCommandBlock::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xCommandBlock(this);
}




void xCommandBlock::print()
{
  cout <<"\n\txCommandBlock_Node:\n";
  if (CommandList)
    {
      for (TIndex index=0; index<~(*CommandList); index++)
	{
	  cout <<"\t";
	  ((*CommandList)[index])->print();
	}
    }
  cout <<"\n";
}





xAssignment::xAssignment(xIdentifier *destination, xExpression *source)
:Destination(destination),Source(source)
{
}




xAssignment::~xAssignment()
{
}




void xAssignment::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xAssignment(this);
}




void xAssignment::print()
{
  cout <<"xAssignment_Node: ";
  if (Destination)
    {
      Destination->print();
    }
  cout << " := ";
  if (Source)
    {
      Source->print();
    }
  cout <<"\n";
}





xWhile::xWhile(xCondition *condition, xCommand *command)
:Condition(condition),Command(command)
{
}




xWhile::~xWhile()
{
}




void xWhile::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xWhile(this);
}




void xWhile::print()
{
  cout <<"xWhile_Node:\n";
  if (Condition)
    {
      cout <<"\t";
      Condition->print();
    }
  if (Command)
    {
      cout <<"\t";
      Command->print();
    }
  cout << "\n";
}





xSum::xSum(xExpression *left, xExpression *right)
:Left(left),Right(right)
{
}




xSum::~xSum()
{
}




void xSum::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xSum(this);
}




void xSum::print()
{
  if (Left)
    {
      Left->print();
    }
  cout <<"+";
  if(Right)
    {
      Right->print();
    }
}




xDiff::xDiff(xExpression *left, xExpression *right)
:Left(left),Right(right)
{
}




xDiff::~xDiff()
{
}




void xDiff::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xDiff(this);
}




void xDiff::print()
{
  if (Left)
    {
      Left->print();
    }
  cout <<"-";
  if(Right)
    {
      Right->print();
    }
}






xConstant::xConstant(xString value)
:Value(value)
{
}




xConstant::~xConstant()
{
}




void xConstant::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xConstant(this);
}




void xConstant::print()
{
  cout << Value;
}




xMemoryObject::xMemoryObject()
:Address(0)
{
}




xMemoryObject::~xMemoryObject()
{
}




xIdentifier::xIdentifier(xString name)
:Name(name), Address(0), Type(Unknown)
{
}




xIdentifier::~xIdentifier()
{
}




xVariable::xVariable(xString name)
:xIdentifier(name)
{
  Type = Simple;
}




xVariable::~xVariable()
{
}




void xVariable::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xVariable(this);
}




TBool xVariable::operator==(xVariable variable)
{
  return (Name==(variable.Name));
}




void xVariable::print()
{
  cout << Name;
}




xArray::xArray(xString name, xExpression *index)
:xIdentifier(name),Index(index)
{
  Type=Array;
}




xArray::~xArray()
{
}




void xArray::AcceptVisitor(xPascallinoVisitor &visitor)
{
  visitor.visit_xArray(this);
}




TBool xArray::operator==(xArray array)
{
  return (Name==(array.Name));
}





void xArray::print()
{
  cout << Name << "[";
  if (Index)
    {
      Index->print();
    }
  cout << "]";  
}

















